/* Mptn -- regexp-like pattern matcher suited for morphological analysis
 * Copyright (C) 2000  George Bronnikov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
  Working with character sets.
  (Turn them into colorsets?)

  $Id: mptn-chrset.c 1.2 Sun, 23 Jan 2000 20:56:51 -0600 goga $
*/
#include <glib.h>

#include "mptn-types.h"
#include "mptn-chrset.h"

/* Number of int bitmask corresponding to c */
#define N_INT(c) ((guint)(guchar)(c) / BITS_IN_INT)

/* Number of bit in that bitmask */
#define N_BIT(c) ((guint)(guchar)(c) % BITS_IN_INT)

/* Initialize an ampty chrset */
mptn_chrset_t *
mptn_chrset_create ()
{
  return g_new0 (gint, INTS_IN_CHRSET);
}

/* Destroy a chrset */
void 
mptn_chrset_destroy (mptn_chrset_t *chrset)
{
  g_return_if_fail (chrset != NULL);

  g_free (chrset);
}

/* Add c to chrset */
void
mptn_chrset_add (mptn_chrset_t *chrset, chr c)
{
  int *cs = chrset;

  g_return_if_fail (cs != NULL);
  
  cs [N_INT(c)] |= 1 << N_BIT(c);
}

/* Remove c from chrset */
void 
mptn_chrset_remove (mptn_chrset_t *chrset, chr c)
{
  int *cs = chrset;

  g_return_if_fail (cs != NULL);

  cs [N_INT(c)] &= ~(1 << N_BIT(c));
}

/* Is c in chrset? */
gboolean 
mptn_chrset_in (mptn_chrset_t *chrset, chr c)
{
  int *cs = chrset;

  g_return_val_if_fail (cs != NULL, FALSE);

  return (cs [N_INT(c)] >> N_BIT(c)) & 0x01;
}

/* Reverse a chrset -- works in place */
void 
mptn_chrset_reverse (mptn_chrset_t *chrset)
{
  int i;
  int *cs = chrset;

  g_return_if_fail (cs != NULL);

  for (i = 0; i < INTS_IN_CHRSET; i++)
    cs[i] = ~cs[i];
}

/*  Copy a chrset */
mptn_chrset_t *
mptn_chrset_copy (mptn_chrset_t *chrset)
{
  return g_memdup (chrset, INTS_IN_CHRSET * sizeof(int));
}
